'PSB TinyFont Driver (C) Rayslogic.com, LLC
'This is a modified version of Parallax's VGA driver, adapted for the LCD of the PSB

CON

  paramcount    = 5  'RJA: Don't need so many parameters for PSB!
  colortable    = $180  'start of colortable inside cog
  
CON
  'Here is where you define the pins you use to control the 4.3" LCD
  'PSB Pins
  '{
  BackLightPin=2
  DisplayOnPin=24
  PixelClockPin=25
  DataEnablePin=26
  TouchIrqPin=27   'This pin is really optional, but touchscreen sets this whenever screen is touched 
  NotUsedPin=31    'Need to define one pin that is not to be used for LCD (can still be used for anything else)
  VgaPins=%010_111  'Pins 16..23  This is the same syntax as used by the VGA driver
  '}
  {
  'Prop Platform Pins
  BackLightPin=24'2
  DisplayOnPin=24   'Note that it is possible to combine BackLight and DisplayOn, just set equal to same pin  
  PixelClockPin=25
  DataEnablePin=26
  TouchIrqPin=27   'This pin is really optional, but touchscreen sets this whenever screen is touched 
  NotUsedPin=31    'Need to define one pin that is not to be used for LCD (can still be used for anything else)
  VgaPins=%010_111  'Pins 16..23  This is the same syntax as used by the VGA driver
  }
  
''  vga_pins
''
''    bits 5..3 select pin group:
''      %000: pins 7..0
''      %001: pins 15..8
''      %010: pins 23..16
''      %011: pins 31..24
''      %100: pins 39..32
''      %101: pins 47..40
''      %110: pins 55..48
''      %111: pins 63..56
''
''    bits 2..0 select top pin within group
''    for example: %01111 (15) will use pins %01000-%01111 (8-15)


  
VAR

  long  cog


PUB start(vgaptr) : okay

'' Start VGA driver - starts a cog
'' returns false if no cog available
''
''   vgaptr = pointer to VGA parameters

  stop
  okay := cog := cognew(@entry, vgaptr) + 1

  

  'Turn on PSB's LCD after a second
  if BacklightPin<>DisplayOnPin 'If not combining these pins
  'Set Pin directions
    OUTA[DisplayOnPin]~
    DIRA[DisplayOnPin]~~
    'Turn on display  
    waitcnt(cnt+clkfreq/4)
    OUTA[DisplayOnPin]~~


PUB stop

'' Stop VGA driver - frees a cog

  if cog
    cogstop(cog~ - 1)


DAT

'********************************
'* Assembly language VGA driver *
'********************************

                        org
'
'
' Entry
'
entry                   mov     taskptr,#tasks          'reset tasks

BL                      or     outa,BL_mask
                        or     dira,BL_mask
                      
                        JMP     #BL_Done 
                        

BL_mask       long    1<<BackLightPin

BL_Done
                        mov     x,#9                    'perform task sections initially
:init                   jmpret  taskret,taskptr
                        djnz    x,#:init
'
'
' Superfield
'
superfield              mov     hv,fixed_hvbase'hvbase               'set hv

'
'
' Field
'
field                   wrlong  visible,par             'set status to visible
                        'movi    vcfg,#%01000_000        '+1, set video configuration
                          'now leaving vcfg in 2-bit color mode for 8x8 tiles! 


:nobl
                        mov     screen,_screen          'point to first tile (upper-leftmost)
                        mov     y,#34'_vt                   'set vertical tiles
                        mov     tileline,#0  
:line                   mov     vx,_vx                  'set vertical expand
:vert  

                        mov     vscl,#16 'hb     hb=16
                        waitvid colortable,#0                    

                        mov     x,#60'_ht                   'set horizontal tiles
                        mov     vscl,fixed_hx'hx                 'set horizontal expand

:tile                   rdword  tile,screen             'read tile info
                                                'upper 4 bits are colors, lower bits are tile base address

                        
                        rol     tile,#4                 'read tile pixels
                        add     tile,tileLine               'set pointer bits into tile  
                                                'upper bits are ignored when doing rdword!
                        rdword  pixels,tile             '(8 clocks between reads)
                        shr     tile,#16              'set tile colors
                        add     tile,#$180 
                        movd    :color,tile
{                                                
                        mov     t1,tile
                        and     t1,#$FF
                        shl     t1,#4
                        add     t1,tileLine          'Factor in current line
                        add     t1,_pFont            'add in font base address

                        'rol     tile,#6                 'read tile pixels
                        rdbyte  pixels,t1             '(8 clocks between reads)
                        shr     tile,#8             'set tile colors
                        add     tile,#$180
                        movd    :color,tile
}                        
                        add     screen,#2               'point to next tile
                                             'Enable Pixel Data

:color                  waitvid colortable,pixels       'pass colors and pixels to video
                        nop
                        nop
                        or      outa,DeMask1     'RJA  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
                        djnz    x,#:tile                'another tile?

                        
                        sub     screen,fixed_hc2x'hc2x             'repoint to first tile in same line

                        mov     vscl,fixed_hf'hf
                        waitvid colortable,#0
                        nop
                        nop
                        and      outa,DeMask0     'RJA  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


                        

                        mov     x,#1                    'do hsync
                        call    #blank_hsync            '(x=0)

:skip                   djnz    vx,#:vert               'vertical expand?
                        add     tileline,#2
                        cmp     tileline,#16 wz,wc'#16 wz,wc
              if_b      jmp     #:line
                        mov     tileline,#0       
                        add     screen,fixed_hc2x'hc2x             'point to first tile in next line
                        djnz    y,#:line        wc      'another tile line? (c=0)


                        movi    vcfg,#%01100_000        '+1, set video configuration 
                        
                        wrlong  invisible,par           'unless interlace and field1, set status to invisible 
                        
                        mov     taskptr,#tasks          'reset tasks

                        addx    x,_vf         '  wc      'do invisible front porch lines (x=0 before, c=0 after)
                        call    #blank_line


              'Do vertical sync
                        xor     hv,#%01
                        waitvid hv,#0
                        mov     vscl,fixed_h2'h2                 'blank line or vsync-normal?
                        xor     hv,#%01
                        waitvid hv,#0

                        mov     x,_vb                   'do invisible back porch lines, except last 
                        call    #blank_line


                        jmp     #superfield             'else, new superfield
'        
'
' Blank line(s)


blank_line              mov     vscl,fixed_h1'h1                 'blank line or vsync-interlace?
                        add     vscl,fixed_h2'h2
bcolor                  waitvid hv,#0

                        jmpret  taskret,taskptr         'call task section (z undisturbed)

blank_hsync             mov     vscl,_hf                'hsync, do invisible front porch pixels
                        waitvid hv,#0

                        mov     vscl,_hs                'do invisble sync pixels
                        xor     hv,#%10
                        waitvid hv,#0

                        mov     vscl,_hb                'do invisible back porch pixels
                        xor     hv,#%10
                        waitvid hv,#0

                        djnz    x,#blank_line 

                     '   movd    bcolor,#hv
blank_hsync_ret
blank_line_ret
                        ret
'
'

'RJA:  !!!!!!!!!!!!!!!!!!!!!!!!!!!!
LcdPins   long 1<<DataEnablePin+1<<PixelClockPin +1<<BackLightPin
DeMask1       long 1<<DataEnablePin'
DeMask0       long (-1)-(1<<DataEnablePin)'

' Tasks - performed in sections during invisible back porch lines
'
tasks                   mov     t1,par                  'load parameters
                        movd    :par,#_enable           '(skip _status)
                        mov     t2,#paramcount - 1
:load                   add     t1,#4
:par                    rdlong  0,t1
                        add     :par,d0
                        djnz    t2,#:load               '+164

                        mov     _pins, PsbPins  'RJA:  refresh pins  !!!!!!!!!!!!!!!!!!!! 
                        
                        mov     t1,#2                   'set video pins and directions
                        shl     t1,_pins                '(if video disabled, pins will drive low)
                        sub     t1,#1
                        test    _pins,#$20      wc
                        and     _pins,#$38
                        shr     t1,_pins
                        movs    vcfg,t1
                        shl     t1,_pins
                        shr     _pins,#3
                        movd    vcfg,_pins
        if_nc           mov     dira,t1
        if_nc           or      dira,LcdPins  'RJA  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        if_nc           mov     dirb,#0
        if_c            mov     dira,#0
        if_c            mov     dirb,t1                 '+14

                        tjz     _enable,#disabled       '+2, disabled?

                        jmpret  taskptr,taskret         '+1=181, break and return later
     
                        movi    ctra,fixed_ctrai't1                 '+22
                        movd    ctra,#PixelClockPin  '       'RJA: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                        movs    ctra,#NotUsedPin  '       'RJA: !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                        
                        mov     TestOut,t2 
                        mov     frqa,fixed_frqa't2                 '+136

                        jmpret  taskptr,taskret         '+1=173, break and return later

                        movi    vcfg,#%01100_000        '+1, set video configuration

:Backlight              jmpret  taskptr,taskret         'add one more task (backlight) 'should be OK with 12 backporch lines...

                        mov     y,_backlight'#30'33
:BL_Loop                nop
                        nop
                        nop
                        nop
                        xor     outa,BL_mask
                        nop
                        nop
                        nop
                        nop
                        or     outa,BL_mask
                        djnz    y,#:BL_Loop                                              

:colors                 jmpret  taskptr,taskret         '+1=114/160, break and return later
      
                        mov     t1,#13                  'load next 13 colors into colortable
:loop                   mov     t2,:color               '5 times = 65 (all 64 colors loaded)
                        shr     t2,#9-2
                        and     t2,#$FC
                        add     t2,_colors
                        rdlong  t2,t2
                        and     t2,colormask
                        or      t2,fixed_hvbase'hvbase
:color                  mov     colortable,t2
                        add     :color,d0
                        andn    :color,d6
                        djnz    t1,#:loop               '+158


                        jmp     #:colors                '+1, keep loading colors
'
'
' Disabled - reset status, nap ~4ms, try again
'
disabled                mov     ctra,#0                 'reset ctra
                        mov     vcfg,#0                 'reset video

                        wrlong  outa,par                'set status to disabled

                        rdlong  t1,#0                   'get CLKFREQ
                        shr     t1,#8                   'nap for ~4ms
                        min     t1,#3
                        add     t1,cnt
                        waitcnt t1,#0

                        jmp     #entry                  'reload parameters
'
'

CtrlaAdd                long  25
DataEnableMask          long 1<<DataEnablePin

' Initialized data
d0                      long    1 << 9 << 0
d6                      long    1 << 9 << 6

invisible               long    1
visible                 long    2
vmask                   long    $01010101
hmask                   long    $02020202
colormask               long    $FCFCFCFC
'

{
_pins                   long     %010_111       '%pppttt        read-only
}
'These two variables need to be set because Chip's code modifies them...
'PixelClock              long     4_000_000       '500_000+       read-only
PsbPins                 long     VgaPins'%000_111'010_111 

_mode                   long     %000       '%tihv          read-only
 
_ht                     long     30       '1+             read-only
_vt                     long     17       '1+             read-only

_hx                     long     1       '1+             read-only
_vx                     long     1       '1+             read-only
_ho                     long     0       '0+-            read-only
_vo                     long     0       '0+-            read-only

_hd                     long     512'512       '1+             read-only
_hf                     long     8       '1+             read-only
_hs                     long     41'96       '1+             read-only
_hb                     long     55'45'48       '1+             read-only
 
_vd                     long     272'288'380       '1+             read-only
_vf                     long     24'11       '1+             read-only
_vs                     long     1'2       '1+             read-only
_vb                     long     13'12'10'31       '2+             read-only


fixed_hx                long    1<<12+8'$1010
fixed_hc2x              long    120'160  '#columns*2
fixed_hf                long    16
fixed_h2                long    209
fixed_h1                long    303
fixed_ctrai             long    27
fixed_frqa             long    $0CCCCCCC
fixed_hvbase            long    $03030303
TileLine                long     0


TestOut                 res     1


' 
' Uninitialized data
'
taskptr                 res     1                       'tasks
taskret                 res     1
t1                      res     1
t2                      res     1

x                       res     1                       'display
y                       res     1
vx                      res     1
screen                  res     1
tile                    res     1
pixels                  res     1

hv                      res     1

'
'
' Parameter buffer
'
_enable                 res     1       '0/non-0        read-only
_screen                 res     1       '@word          read-only
_colors                 res     1       '@long          read-only
_backlight              res     1       'backlight level request
_pFont                  res     1       'Font address


'     
_rate                   res     1       'RJA:  copying this from PixelClock variable, since it gets modified by the code
_pins                   res     1       'RJA:  This one also gets a localcopy, PsbPins  
                        fit     colortable              'fit underneath colortable ($180-$1BF)


''
''___
''VAR                   'VGA parameters - 21 contiguous longs
''
''  long  vga_status    '0/1/2 = off/visible/invisible      read-only
''  long  vga_enable    '0/non-0 = off/on                   write-only
''  long  vga_pins      '%pppttt = pins                     write-only
''  long  vga_mode      '%tihv = tile,interlace,hpol,vpol   write-only
''  long  vga_screen    'pointer to screen (words)          write-only
''  long  vga_colors    'pointer to colors (longs)          write-only            
''  long  vga_ht        'horizontal tiles                   write-only
''  long  vga_vt        'vertical tiles                     write-only
''  long  vga_hx        'horizontal tile expansion          write-only
''  long  vga_vx        'vertical tile expansion            write-only
''  long  vga_ho        'horizontal offset                  write-only
''  long  vga_vo        'vertical offset                    write-only
''  long  vga_hd        'horizontal display ticks           write-only
''  long  vga_hf        'horizontal front porch ticks       write-only
''  long  vga_hs        'horizontal sync ticks              write-only
''  long  vga_hb        'horizontal back porch ticks        write-only
''  long  vga_vd        'vertical display lines             write-only
''  long  vga_vf        'vertical front porch lines         write-only
''  long  vga_vs        'vertical sync lines                write-only
''  long  vga_vb        'vertical back porch lines          write-only
''  long  vga_rate      'tick rate (Hz)                     write-only
''
''The preceding VAR section may be copied into your code.
''After setting variables, do start(@vga_status) to start driver.
''
''All parameters are reloaded each superframe, allowing you to make live
''changes. To minimize flicker, correlate changes with vga_status.
''
''Experimentation may be required to optimize some parameters.
''
''Parameter descriptions:
''  __________
''  vga_status
''
''    driver sets this to indicate status:
''      0: driver disabled (vga_enable = 0 or CLKFREQ < 16MHz)
''      1: currently outputting invisible sync data
''      2: currently outputting visible screen data
''  __________
''  vga_enable
''
''        0: disable (pins will be driven low, reduces power)
''    non-0: enable
''  ________
''  vga_pins
''
''    bits 5..3 select pin group:
''      %000: pins 7..0
''      %001: pins 15..8
''      %010: pins 23..16
''      %011: pins 31..24
''      %100: pins 39..32
''      %101: pins 47..40
''      %110: pins 55..48
''      %111: pins 63..56
''
''    bits 2..0 select top pin within group
''    for example: %01111 (15) will use pins %01000-%01111 (8-15)
''  ________
''  vga_mode
''
''    bit 3 selects between 16x16 and 16x32 pixel tiles:
''      0: 16x16 pixel tiles (tileheight = 16)
''      1: 16x32 pixel tiles (tileheight = 32)
''
''    bit 2 controls interlace:
''      0: progressive scan (less flicker, good for motion, required for LCD monitors)
''      1: interlaced scan (allows you to double vga_vt, good for text)
''
''    bits 1 and 0 select horizontal and vertical sync polarity, respectively
''      0: active low
''      1: active high
''  __________
''  vga_screen
''
''    pointer to words which define screen contents (left-to-right, top-to-bottom)
''      number of words must be vga_ht * vga_vt
''      each word has two bitfields: a 6-bit colorset ptr and a 10-bit pixelgroup ptr
''        bits 15..10: select the colorset* for the associated pixel tile
''        bits 9..0: select the pixelgroup** address %ppppppppppcccc00 (p=address, c=0..15)
''
''       * colorsets are longs which each define four 8-bit colors
''
''      ** pixelgroups are <tileheight> longs which define (left-to-right, top-to-bottom) the 2-bit
''         (four color) pixels that make up a 16x16 or a 16x32 pixel tile
''  __________
''  vga_colors
''
''    pointer to longs which define colorsets
''      number of longs must be 1..64
''      each long has four 8-bit fields which define colors for 2-bit (four color) pixels
''      first long's bottom color is also used as the screen background color
''      8-bit color fields are as follows:
''        bits 7..2: actual state of pins 7..2 within pin group*
''        bits 1..0: don't care (used within driver for hsync and vsync)
''
''    * it is suggested that:
''        bits/pins 7..6 are used for red
''        bits/pins 5..4 are used for green
''        bits/pins 3..2 are used for blue
''      for each bit/pin set, sum 240 and 470-ohm resistors to form 75-ohm 1V signals
''      connect signal sets to RED, GREEN, and BLUE on VGA connector
''      always connect group pin 1 to HSYNC on VGA connector via 240-ohm resistor
''      always connect group pin 0 to VSYNC on VGA connector via 240-ohm resistor
''  ______
''  vga_ht
''
''    horizontal number of pixel tiles - must be at least 1
''  ______
''  vga_vt
''
''    vertical number of pixel tiles - must be at least 1
''  ______
''  vga_hx
''
''    horizontal tile expansion factor - must be at least 1
''
''    make sure 16 * vga_ht * vga_hx + ||vga_ho is equal to or at least 16 less than vga_hd
''  ______
''  vga_vx
''
''    vertical tile expansion factor - must be at least 1
''
''    make sure <tileheight> * vga_vt * vga_vx + ||vga_vo does not exceed vga_vd
''      (for interlace, use <tileheight> / 2 * vga_vt * vga_vx + ||vga_vo)
''  ______
''  vga_ho
''
''    horizontal offset in ticks - pos/neg value (0 recommended)
''    shifts the display right/left
''  ______
''  vga_vo
''
''    vertical offset in lines - pos/neg value (0 recommended)
''    shifts the display up/down
''  ______
''  vga_hd
''
''    horizontal display ticks
''  ______
''  vga_hf
''
''    horizontal front porch ticks
''  ______
''  vga_hs
''
''    horizontal sync ticks
''  ______
''  vga_hb
''
''    horizontal back porch ticks
''  ______
''  vga_vd
''
''    vertical display lines
''  ______
''  vga_vf
''
''    vertical front porch lines
''  ______
''  vga_vs
''
''    vertical sync lines
''  ______
''  vga_vb
''
''    vertical back porch lines
''  ________
''  vga_rate
''
''    tick rate in Hz
''
''    driver will limit value to be within 500KHz and 128MHz
''    pixel rate (vga_rate / vga_hx) should be no more than CLKFREQ / 4