'PSB WiflyWeb       Copyright (C) 2010 Rayslogic.com, LLC
'Testing wireless web page download using Wifly GSX mounted on PSB accessory board

CON      'System Constants

  _clkmode = xtal1+pll16x
  _clkfreq = 80_000_000


  'LCD is 480x272 giving 60x34 of 8x8 tiles  (17 rows of text)
  cols = 60
  rows = 17*2  
  screensize = cols * rows
  fontsize=4080  '#bytes in font data = 255*16
  
  WebPageBufferSize=10000
  
VAR    'System Variables

  long  lcd_status      'status: off/visible/invisible  read-only       (5 contiguous longs)
  long  lcd_enable      'enable: off/on                 write-only
  long  lcd_videobase   'video base @word               write-only
  long  lcd_colorbase   'color base @long               write-only
  long  lcd_backlight   'Used to control LCD backlight
'  long  lcd_pFont       'Used to pass address of font

  word  screen[screensize]
  long  col, row, color
  long  boxcolor,ptr

  word  pFont

  byte WebPageBuffer[WebPageBufferSize]

DAT       'Data for user font
padding LONG  "A"[4] 'alignment padding for the following user defined characters
TileFont long
file "8x8_tilefont.dat"


OBJ
  lcd : "PSB_8x8_TiledLcdDriver"  'Modified to drive 4.3" TFT
  i2c: "PSB_I2CDriver"
  ser:  "FullDuplexSerialExtended" 

pub Main|i,x,y,z, LoopCount

  'First, need to prepare user font for display (need to move so that last four bits of address are all zeroes!)
  '16 byte align the user font
  pFont := @TileFont & $FFF0 'destination
  bytemove(pFont,@TileFont,fontsize)
  pFont>>=4  'shift font pointer four bits right for ease of use later
    
  'Start up Display
  'clear screen
  print($100)  
  'start driver
  lcd_enable:=1
  lcd_videobase := @screen
  lcd_colorbase := @vgacolors
  lcd_backlight:=16  '1=bright, 16=medium, 32=dark
  'lcd_pFont:=@TileFont 'pass address of font
  lcd.start(@lcd_status)

  i2c.initialize

  'Connect to Wifly at default baud rate
  ser.start(9,8,0,9600)       '  <<<<<<<<<<<<<<<<<<<<  Using Pins 8&9 for Wifly GSX


  'Establish web connection and display progress (just this first time through)
  ConnectInternet
  OpenWebServer
   
  LoopCount:=0 'reset loop counter


  repeat 'Get and show weather loop  
    LoopCount++ 
    'Capture weather info web page  from government web server 
    GetWebPage
    'Parse and display weather
    if ParseWeatherWebPage==false
      printStr(String("Process Web Page Failed."))

    'Close connection without showing anything on screen
    EnterCommandModeSilent
    ser.str(string("close",13,10))

    'Show loop number
    row:=20
    col:=0
    PrintStr(string("Loop# "))
    dec(LoopCount)

    'wait a minute or two
    waitcnt(cnt+clkfreq*60*2)

    'Connect again but this time don't show anything
    ser.str(string("close",13,10))
    waitcnt(cnt+clkfreq/2)
    ser.str(string("ping g",13,10))
    waitcnt(cnt+clkfreq/2)
    ser.str(string("open mobile.weather.gov 80",13,10))
    waitcnt(cnt+clkfreq)
    repeat until ser.rxcheck<0
    
     
  repeat

PRI ConnectInternet  'Connect Wifly to internet
  'Wait a second to let Wifly warm up
  waitcnt (cnt+clkfreq)

  EnterCommandMode

  'Test web connection by pinging the gateway
  SendCommand(string("ping g"))
  ShowResponse(2000)

  'Disable the Wifly automated connect response
  SendCommand(string("set c r 0"))
  ShowResponse(1000)

  'Close tcp connection, just in case left open
  SendCommand(string("close",13,10))
  ShowResponse(1000)

PRI EnterCommandMode
  'Send command code
  repeat 3
    ser.tx("$")
    waitcnt(cnt+clkfreq/4)
  ser.tx(13)
  ser.tx(10)
  ShowResponse(1000)
  
PRI EnterCommandModeSilent
  'Send command code
  repeat 3
    ser.tx("$")
    waitcnt(cnt+clkfreq/4)
  ser.tx(13)
  ser.tx(10)
  

  
Pri OpenWebServer    'open server
  'Connect to government web server
  SendCommand(string("open mobile.weather.gov 80",13,10))
  ShowResponse(1000)


PRI GetWebPage      'retrieve weather web page
  ser.rxflush

  'Issue get command to government weather web server
  SendCommand(string("GET /ob.php?obvar=KIAD  HTTP/1.1",13,10,"Host: mobile.weather.gov",13,10,13,10))
  StoreResponse(5000)
  

PRI ParseWeatherWebPage:bOK|pos1,pos2,pSearch,i,j,k  'Extract weather info from returned web page  
  'clear screen
  print($100)

  'repeat i from 0 to WebPageBufferSize-1
  'printStr(@WebPageBuffer)
  pos1:=0
  
  'Find start of location string, comes after: "Current Local Conditions at:<br>"
  pSearch:=string("at:<br>")
  pos1:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos1==false
    return false  'couldn't find string
  pos1+=StrSize(pSearch)

  'Find End of location string
  pSearch:=string("<br>")
  pos2:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos2==false
    return false  'couldn't find string    

  'Print Location
  repeat i from pos1+1 to pos2-1
    print(WebPageBuffer[i])
  print(13)

  'Print next 3 lines
  repeat 3

    'Weather is next
    pSearch:=string("<br>")
    pos1:=pos2+1'SeekString(pos2+2,pSearch,@WebPageBuffer)
     
    pSearch:=string("<br>")                                        
    pos2:=SeekString(pos1+2,pSearch,@WebPageBuffer)
     
    'Print Weather
    repeat i from pos1+3 to pos2-1
      print(WebPageBuffer[i])
    print(13)

    pos2++

                                   

  '
  pSearch:=string("Temperature:")
  pos1:=SeekString(pos2,pSearch,@WebPageBuffer)
  if pos1==false
    return false  'couldn't find string
  'pos1+=StrSize(pSearch)

  'Find End of temperature string
  pSearch:=string("&")
  pos2:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos2==false
    return false  'couldn't find string
  
  'Print Fahrenheit Temperature
  repeat i from pos1 to pos2-1
    print(WebPageBuffer[i])
  printStr(string(176,"F, "))

  'Find end of Celsius
  pos1:=pos2+8
  pSearch:=string("&")
  pos2:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos2==false
    return false  'couldn't find string

  'Print Celsius Temperature
  repeat i from pos1 to pos2-1
    print(WebPageBuffer[i])
  printStr(string(176,"C"))  
  print(13)

  'Humidity is next
  pSearch:=string("<br>")
  pos1:=SeekString(pos2,pSearch,@WebPageBuffer)
  if pos1==false
    return false  'couldn't find string
  pos1+=StrSize(pSearch)
  
  pSearch:=string("<br>")
  pos2:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos2==false
    return false  'couldn't find string
  
  'Print Humidity
  repeat i from pos1+1 to pos2-1
    print(WebPageBuffer[i])
  print(13)

  'Wind speed is next
  pSearch:=string("<br>")
  pos1:=SeekString(pos2,pSearch,@WebPageBuffer)
  if pos1==false
    return false  'couldn't find string
  pos1+=StrSize(pSearch)
  
  pSearch:=string("<br>")
  pos2:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos2==false
    return false  'couldn't find string
  
  'Print Windspeed
  repeat i from pos1+1 to pos2-1
    print(WebPageBuffer[i])
  print(13)

  'Barometer is next
  pSearch:=string("<br>")
  pos1:=SeekString(pos2,pSearch,@WebPageBuffer)
  if pos1==false
    return false  'couldn't find string
  pos1+=StrSize(pSearch)
  
  pSearch:=string("<br>")
  pos2:=SeekString(pos1,pSearch,@WebPageBuffer)
  if pos2==false
    return false  'couldn't find string
  
  'Barometer Windspeed
  repeat i from pos1+1 to pos2-1
    print(WebPageBuffer[i])
  print(13)

  
  return true
  
Pri SeekString(pos, pSeekStr, pSearchStr):posFound|i,j,k,m,n  'Find position of one string within another 
  m:=0
  repeat
    'get first character we're looking for
    i:=byte[pSeekStr][m++]      
    if i==0
      return 0 'searching for a null string

    repeat 'Find first character
      j:=byte[pSearchStr][pos++]
      if j==0
        return false  'reached end of search string with no match
    until i==j
    
    'now see if remaining characters match
    n:=pos
    repeat
      i:=byte[pSeekStr][m++]
      if i==0
        pos-- 'want to return position of first character in search string
        return pos 'all done
      j:=byte[pSearchStr][n++]
      if j==0
        return false
    until i<>j             
    'Start over
    m:=0      
    
    
    


PRI SendCommand(pStr)  'Send command and show it
color:=1
print(13)
printStr(pStr)
print(13)
color:=0
ser.str(pStr)
ser.tx(13)

PRI StoreResponse(ms)|i,wcnt,j   
  wcnt:=cnt+ms*(clkfreq/1000)
  j:=0                                   
  repeat
    'Check for byte in serial input buffer
    i:=ser.rxcheck
    if i>0
      'Store gotten byte in web page buffer
      WebPageBuffer[j]:=i
      j++
      'Make sure not to overrun input buffer
      if j=>WebPageBufferSize
        WebPageBuffer[WebPageBufferSize-1]:=0
        quit
    else
      'Check for timeout
      if (wcnt-cnt)<0
        WebPageBuffer[j]:=0
        quit


PRI ShowResponse(ms)|i,wcnt,j   
  wcnt:=cnt+ms*(clkfreq/1000)
  repeat
    i:=ser.rxcheck
    if i>0
      if i==$0D
        ser.rx
      'if j++<500
      print(i)
      'if j++>500
      '  quit
    else
      if (wcnt-cnt)<0
        quit

PUB WaitStylusUp
  'Wait for stylus to be up
  repeat
    waitcnt(cnt+clkfreq/100)  
    if ina[lcd#TouchIrqPin]<>0
    return


PRI PrintStr(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    print(byte[stringptr++])

PUB print(c) | i, k

  case c
    $0D:               'return?
      newline
    $0A:
      'lf
    
    $00..$FF:           'character?
      k := color << 1  'text colors are defined in pairs... 
      screen[row * cols + col] := (k+1) << 12 + pFont + c  
      screen[(row + 1) * cols + col] := k << 12 + pFont + c  
      if ++col == cols
        newline

    $100:               'clear screen?
      wordfill(@screen, pFont+32, screensize)
      col := row := 0

    '$108:               'backspace?
    '  if col
    '    col--



    '$110..$11F:         'select color?
    '  color := c & $F


' New line

PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
    wordfill(@screen[(rows-2)*cols], pFont+32, cols<<1)


' Data

PRI dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    print("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      print(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      print("0")
    i /= 10

PRI hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    print(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
    
DAT 'Default Touchscreen Calibrations
touchx0 long 2713
touchdx long 123
touchy0 long 4590
touchdy long 206

finder byte "X"[100]



DAT
                    'Note:  Can only use 16 longs of colors with this driver...
vgacolors               long
                        long    $F00CF00C       '0blue background with yellow letters
                        long    $F0F00C0C       '1
                        long    $08A808A8       '2green
                        long    $0808A8A8       '3
                        long    $50005000       '4blue
                        long    $50500000       '5
                        long    $FC00FC00       '6white
                        long    $FCFC0000       '7
                        long    $FF80FF80       '8red/white
                        long    $FFFF8080       '9
                        long    $FFC03000      '10'Colors for Graphics
spcl                    long    $F0FF9C0C       '11 'Color0  white
                        long    $F0C09C0C        '12 Color1  red
                        long    $F030000C      '13 Color2  green
                        long    $F000000C      '14 Color3  black
                        long    $FF0C9C0C        '15 fill screen
